﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Remoting.Proxies;
using System.Runtime.Remoting.Messaging;
using System.Runtime.Remoting.Activation;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Services;

namespace Monk.Aop.Proxy
{
    internal class AspectProxy : RealProxy	
    {
        /// <summary>
        /// 被代理的对象
        /// </summary>
        private MarshalByRefObject target;
        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="myType">被代理的类的类型</param>
        /// <param name="obj">被代理的对象</param>
        public AspectProxy(Type myType, MarshalByRefObject obj): base(myType)
        {
            target = obj;
        }
        /// <summary>
        /// 当在派生类中重写时，在当前实例所表示的远程对象上调用在所提供的 IMessage 中指定的方法
        /// 在这里执行对方法执行的拦截处理
        /// </summary>
        /// <param name="msg">IMessage，包含有关方法调用的信息。</param>
        /// <returns>调用的方法所返回的消息，包含返回值和所有 out 或 ref 参数。</returns>
        public override IMessage Invoke(IMessage msg)
        {
            IMethodCallMessage call = msg as IMethodCallMessage;
            //通过AspectManager获得这个方法的所有拦截方法
             IAspectAdvice[] myAspect = AspectManager.FindAspect(this.GetProxiedType(),
                 msg as IMethodMessage);
            IMessage retMsg;
            if (myAspect != null)
            {
                foreach (IAspectAdvice advice in myAspect)
                {
                    //执行前通知方法
                    advice.BeforeExecute(call);
                }
            }
            //构造对象
            if (msg is IConstructionCallMessage)
            {
                IConstructionCallMessage ccm = (IConstructionCallMessage)msg;
                RemotingServices.GetRealProxy(target).InitializeServerObject(ccm);
                ObjRef oRef = RemotingServices.Marshal(target);
                RemotingServices.Unmarshal(oRef);
                retMsg = EnterpriseServicesHelper.CreateConstructionReturnMessage(ccm, 
                    (MarshalByRefObject)this.GetTransparentProxy());
            }
            else
            {
                //一般方法
                IMethodCallMessage mcm = (IMethodCallMessage)msg;
                retMsg = RemotingServices.ExecuteMessage(target, mcm);
            }
            if (myAspect != null)
            {
                foreach (IAspectAdvice advice in myAspect)
                {
                    //尾通知方法
                    advice.AfterExecute(retMsg as IMethodReturnMessage);
                }
            }
            return retMsg;
        }
    }
}
